% Risk neutral extraction from implied volatilities.

clc; clear;

% Inline function for the Black Scholes call
BSCall = inline('s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T))',...
	     's','K','r','q','v','T');

% Method 1: Estimated alpha, rho, and volvol directly.
% Method 2: Estimate rho and volvol, and obtain alpha from the ATM vol.
SABRmethod = 1;

% Input Strikes and Implied vols for Google, and option features
K = [300 310 320 330 340 350 360 370 380 390 400 400 410 410 ...
	 420 420 430 430 440 440 450 450 460 470 480 490 500 510];

IV = [0.6842 0.6566 0.6197 0.5928 0.5311 0.5205 0.4838 0.4669 0.4420 0.4267 0.4027...
	  0.4120 0.3974 0.4019 0.3894 0.3918 0.3783 0.3815 0.3714 0.3802 0.3643 0.3663...
	  0.3653 0.3730 0.3705 0.3857 0.3783 0.4006];

% Find the unique values of K and corresponding IV
[x,y,z] = (unique(K));
K = K(y);
IV = IV(y);

% Option and stock price parameters Parameters
T  = 0.04;       % Maturity
rf = 0.01;       % Risk free rate
S  = 423.19;     % Spot price

% Find the ATM volatility (SABR model only)
ATMvol = interp1(K,IV,S,'cubic');

% Fit the DVF
options = optimset('MaxFunEvals', 1e5, 'MaxIter', 1e5);
beta = regstats(IV, K, 'purequadratic');
yhat = beta.yhat;
B = beta.beta;

% Fit the SVI
start = 0.5.*[1 1 1 1 1];
[params feva] = fminsearch(@(b) findSVIparams(b,K,IV,S,rf,T), start,options);
a = params(1);
b = params(2);
r = params(3);
m = params(4);
s = params(5);

% Fit the SABR
beta = 0.5;
if SABRmethod == 1;
	start = [.3 .3];
	[param, feval] = fminsearch(@(par)EstimateRhoAndVol(par, K, IV, ATMvol, S, T, beta), start, options);
	rho    = param(1);
	volvol = param(2);
	alpha  = findAlpha(S,S,T,ATMvol,beta,rho,volvol);
elseif SABRmethod == 2;
	start = [.3 .3 .2];
	[param, feval] = fminsearch(@(par)EstimateAllParameters(par, K, IV, S, T, beta), start, options);
	alpha  = param(1);
	rho    = param(2);
	volvol = param(3);
end

% Convert to Black Scholes call prices
% Extrapolate the wings of the IV using DVF or SVI tails
% The strikes go from 300 to 510, extrapolate to 250 and 560.
% Range of strikes to fit.
dK = 0.25;
K2 = [250:dK:560];

% Fitted volatilities and Black Scholes prices.
% Flat extrapolation for the interpolated IV
for i=1:length(K2)
		DVF_vol(i) = B(1) + B(2)*K2(i) + B(3)*K2(i)^2;
		SVI_vol(i) = SVI(K2(i),S,rf,T,a,b,r,m,s);
		SABR_vol(i) = SABRvol(alpha,beta,rho,volvol,S,K2(i),T);
		BSC_DVF(i)  = BSCall(S,K2(i),rf,0,DVF_vol(i),T);
		BSC_SVI(i)  = BSCall(S,K2(i),rf,0,SVI_vol(i),T);
		BSC_SABR(i) = BSCall(S,K2(i),rf,0,SABR_vol(i),T);
		if K2(i) <= min(K);
			INT_vol(i) = IV(1);
		elseif K2(i) >= max(K);
			INT_vol(i) = IV(end);
		else
			INT_vol(i) = interp1(K,IV,K2(i),'splines');
		end
		BSC_INT(i) = BSCall(S,K2(i),rf,0,INT_vol(i),T);
end

% Plot the Implied volatilities.
plot(K, IV, 'x', K2, DVF_vol, K2, SVI_vol, K2, SABR_vol, K2, INT_vol)
legend('Market Implied Vol', 'DVF IV', 'SVI IV', 'SABR IV', 'Interpolated IV') 
xlabel('Strike Price')
ylabel('Implied Volatility')
clear beta yhat B
pause

% Plot the call prices using the fitted implied volatility.
plot(K2, BSC_DVF, K2, BSC_SVI, K2, BSC_SABR, K2, BSC_INT)
legend('DVF', 'SVI', 'SABR', 'Interpolated')
xlabel('Strike Price')
ylabel('Call Price')
pause

% Step 6.
% Extract the RND.
[RND_DVF , K3] = ExtractRND(K2,BSC_DVF ,'cubic');
[RND_SVI , K3] = ExtractRND(K2,BSC_SVI ,'cubic');
[RND_SABR, K3] = ExtractRND(K2,BSC_SABR,'cubic');
[RND_INT , K3] = ExtractRND(K2,BSC_INT ,'cubic');

% Plot the RND.  The lognormal tails are a little thin.
plot(K3, RND_DVF, K3, RND_SVI, K3, RND_SABR, K3, RND_INT)
legend('RND from DVF', 'RND from SVI', 'RND from SABR', 'RND from Interpolation');
xlabel('Strike Price')
ylabel('RND')
pause

% Verify the integral is 1.
AreaUnderRND_DVF  = trapz(K2(1:length(RND_DVF)),  RND_DVF)
AreaUnderRND_SVI  = trapz(K2(1:length(RND_SVI)),  RND_SVI)
AreaUnderRND_SABR = trapz(K2(1:length(RND_SABR)), RND_SABR)
AreaUnderRND_INT  = trapz(K2(1:length(RND_INT )), RND_INT )
 
% Verify that the RND recovers the call prices.
for i=1:length(K);
    Kstar = K(i);
    I = find(K3<=Kstar);
	NewStrike = K3;
	NewStrike(I) = 0;
    integrand_DVF  = max(NewStrike - Kstar,0).*RND_DVF;
    integrand_SVI  = max(NewStrike - Kstar,0).*RND_SVI;
    integrand_SABR = max(NewStrike - Kstar,0).*RND_SABR;
    integrand_INT  = max(NewStrike - Kstar,0).*RND_INT;	
    RecoveredCall_DVF(i)  = exp(-rf*T)*trapz(integrand_DVF).*dK;
    RecoveredCall_SVI(i)  = exp(-rf*T)*trapz(integrand_SVI).*dK;
    RecoveredCall_SABR(i) = exp(-rf*T)*trapz(integrand_SABR).*dK;
    RecoveredCall_INT(i)  = exp(-rf*T)*trapz(integrand_INT).*dK;
	MarketCall(i) = BSCall(S,K(i),rf,0,IV(i),T);
	PercentageError_DVF(i)  = (RecoveredCall_DVF(i)  - MarketCall(i))/MarketCall(i)*100;
	PercentageError_SVI(i)  = (RecoveredCall_SVI(i)  - MarketCall(i))/MarketCall(i)*100;
	PercentageError_SABR(i) = (RecoveredCall_SABR(i) - MarketCall(i))/MarketCall(i)*100;
	PercentageError_INT(i)  = (RecoveredCall_INT(i)  - MarketCall(i))/MarketCall(i)*100;
end;
CallChecks = [MarketCall' RecoveredCall_DVF'   RecoveredCall_SVI'   RecoveredCall_SABR'   RecoveredCall_INT' ...
	                      PercentageError_DVF' PercentageError_SVI' PercentageError_SABR' PercentageError_INT']

% Plot recovered prices
plot(K, MarketCall,'x',K,RecoveredCall_DVF,'x',K,RecoveredCall_SVI,'x',K,RecoveredCall_SABR,'x',K,RecoveredCall_INT,'x')
legend('Market Prices', 'Prices using DVF IV', 'Prices using SVI IV', 'Prices using SABR IV', 'Prices using Interpolated IV');
xlabel('Strike Prices');
ylabel('Call Prices')

% Regression R2 to show how close market and recovered calls are.
RegCalls_DVF  = regstats(RecoveredCall_DVF , MarketCall, 'linear');
RegCalls_SVI  = regstats(RecoveredCall_SVI , MarketCall, 'linear');
RegCalls_SABR = regstats(RecoveredCall_SABR, MarketCall, 'linear');
RegCalls_INT  = regstats(RecoveredCall_INT , MarketCall, 'linear');

R2_DVF  = RegCalls_DVF.rsquare
R2_SVI  = RegCalls_SVI.rsquare
R2_SABR = RegCalls_SABR.rsquare
R2_INT  = RegCalls_INT.rsquare

